@@ -9,4 +9,6 @@ __author_email__ = "charlie.clark@clark-consulting.eu"
 __license__ = "MIT"
 __maintainer_email__ = "openpyxl-users@googlegroups.com"
 __url__ = "https://openpyxl.readthedocs.io"
-__version__ = "3.0.6"
+__version__ = "3.1.0b1"
+__version__ = "3.0.8"
+__python__ = "3.6"
