@@ -1,15 +1,14 @@
-from __future__ import absolute_import
-# Copyright (c) 2010-2019 openpyxl
+# Copyright (c) 2010-2021 openpyxl
 
 """Reader for a single worksheet."""
+from copy import copy
 from warnings import warn
 
 # compatibility imports
-from openpyxl.compat import long
 from openpyxl.xml.functions import iterparse
 
 # package imports
-from openpyxl.cell import Cell
+from openpyxl.cell import Cell, MergedCell
 from openpyxl.cell.text import Text
 from openpyxl.worksheet.dimensions import (
     ColumnDimension,
@@ -30,6 +29,7 @@ from openpyxl.utils import (
 from openpyxl.utils.datetime import from_excel, from_ISO8601, WINDOWS_EPOCH
 from openpyxl.descriptors.excel import ExtensionList
 
+from .formula import DataTableFormula, ArrayFormula
 from .filters import AutoFilter
 from .header_footer import HeaderFooter
 from .hyperlink import HyperlinkList
@@ -73,29 +73,31 @@ COL_BREAK_TAG = '{%s}colBreaks' % SHEET_MAIN_NS
 SCENARIOS_TAG = '{%s}scenarios' % SHEET_MAIN_NS
 DATA_TAG = '{%s}sheetData' % SHEET_MAIN_NS
 DIMENSION_TAG = '{%s}dimension' % SHEET_MAIN_NS
+CUSTOM_VIEWS_TAG = '{%s}customSheetViews' % SHEET_MAIN_NS
 
 
 def _cast_number(value):
     "Convert numbers as string to an int or float"
     if "." in value or "E" in value or "e" in value:
         return float(value)
-    return long(value)
+    return int(value)
 
 
 class WorkSheetParser(object):
 
     def __init__(self, src, shared_strings, data_only=False,
-                 epoch=WINDOWS_EPOCH, date_formats=set()):
-        self.min_row = self.min_col = self.max_row = self.max_column = None
+                 epoch=WINDOWS_EPOCH, date_formats=set(),
+                 timedelta_formats=set()):
+        self.min_row = self.min_col = None
         self.epoch = epoch
         self.source = src
         self.shared_strings = shared_strings
         self.data_only = data_only
         self.shared_formulae = {}
-        self.array_formulae = {}
-        self.max_row = self.max_column = 0
+        self.row_counter = self.col_counter = 0
         self.tables = TablePartList()
         self.date_formats = date_formats
+        self.timedelta_formats = timedelta_formats
         self.row_dimensions = {}
         self.column_dimensions = {}
         self.number_formats = []
@@ -104,7 +106,8 @@ class WorkSheetParser(object):
         self.formatting = []
         self.legacy_drawing = None
         self.merged_cells = None
-        self.page_breaks = []
+        self.row_breaks = RowBreak()
+        self.col_breaks = ColBreak()
 
 
     def parse(self):
@@ -116,6 +119,7 @@ class WorkSheetParser(object):
             LEGACY_TAG: self.parse_legacy,
             ROW_BREAK_TAG: self.parse_row_breaks,
             COL_BREAK_TAG: self.parse_col_breaks,
+            CUSTOM_VIEWS_TAG: self.parse_custom_views,
                       }
 
         properties = {
@@ -132,9 +136,10 @@ class WorkSheetParser(object):
             TABLE_TAG: ('tables', TablePartList),
             HYPERLINK_TAG: ('hyperlinks', HyperlinkList),
             MERGE_TAG: ('merged_cells', MergeCells),
+
         }
 
-        it = iterparse(self.source)
+        it = iterparse(self.source) # add a finaliser to close the source when this becomes possible
 
         for _, element in it:
             tag_name = element.tag
@@ -172,7 +177,6 @@ class WorkSheetParser(object):
     def parse_cell(self, element):
         data_type = element.get('t', 'n')
         coordinate = element.get('r')
-        self.max_column += 1
         style_id = element.get('s', 0)
         if style_id:
             style_id = int(style_id)
@@ -184,8 +188,10 @@ class WorkSheetParser(object):
 
         if coordinate:
             row, column = coordinate_to_tuple(coordinate)
+            self.col_counter = column
         else:
-            row, column = self.max_row, self.max_column
+            self.col_counter += 1
+            row, column = self.row_counter, self.col_counter
 
         if not self.data_only and element.find(FORMULA_TAG) is not None:
             data_type = 'f'
@@ -197,9 +203,11 @@ class WorkSheetParser(object):
                 if style_id in self.date_formats:
                     data_type = 'd'
                     try:
-                        value = from_excel(value, self.epoch)
-                    except ValueError:
-                        msg = """Cell {0} is marked as a date but the serial value {1} is outside the limits for dates. The cell will be treated as an error.""".format(coordinate, value)
+                        value = from_excel(
+                            value, self.epoch, timedelta=style_id in self.timedelta_formats
+                        )
+                    except (OverflowError, ValueError):
+                        msg = f"""Cell {coordinate} is marked as a date but the serial value {value} is outside the limits for dates. The cell will be treated as an error."""
                         warn(msg)
                         data_type = "e"
                         value = "#VALUE!"
@@ -234,7 +242,7 @@ class WorkSheetParser(object):
             value += formula.text
 
         if formula_type == "array":
-            self.array_formulae[coordinate] = dict(formula.attrib)
+            value = ArrayFormula(ref=formula.get('ref'), text=value)
 
         elif formula_type == "shared":
             idx = formula.get('si')
@@ -244,6 +252,9 @@ class WorkSheetParser(object):
             elif value != "=":
                 self.shared_formulae[idx] = Translator(value, coordinate)
 
+        elif formula_type == "dataTable":
+            value = DataTableFormula(**formula.attrib)
+
         return value
 
 
@@ -258,26 +269,34 @@ class WorkSheetParser(object):
         attrs = dict(row.attrib)
 
         if "r" in attrs:
-            self.max_row = int(attrs['r'])
+            try:
+                self.row_counter = int(attrs['r'])
+            except ValueError:
+                val = float(attrs['r'])
+                if val.is_integer():
+                    self.row_counter = int(val)
+                else:
+                    raise ValueError(f"{attrs['r']} is not a valid row number")
         else:
-            self.max_row += 1
-        keys = set(attrs)
-        for key in keys:
-            if key.startswith('{'):
-                del attrs[key]
+            self.row_counter += 1
+        self.col_counter = 0
 
-        keys = set(attrs)
-        if keys != set(['r', 'spans']) and keys != set(['r']):
+        keys = {k for k in attrs if not k.startswith('{')}
+        if keys - {'r', 'spans'}:
             # don't create dimension objects unless they have relevant information
-            self.row_dimensions[attrs['r']] = attrs
+            self.row_dimensions[str(self.row_counter)] = attrs
 
         cells = [self.parse_cell(el) for el in row]
-        return self.max_row, cells
+        return self.row_counter, cells
 
 
     def parse_formatting(self, element):
-        cf = ConditionalFormatting.from_tree(element)
-        self.formatting.append(cf)
+        try:
+            cf = ConditionalFormatting.from_tree(element)
+            self.formatting.append(cf)
+        except TypeError as e:
+            msg = f"Failed to load a conditional formatting rule. It will be discarded. Cause: {e}"
+            warn(msg)
 
 
     def parse_sheet_protection(self, element):
@@ -303,12 +322,19 @@ class WorkSheetParser(object):
 
     def parse_row_breaks(self, element):
         brk = RowBreak.from_tree(element)
-        self.page_breaks.append(brk)
+        self.row_breaks = brk
 
 
     def parse_col_breaks(self, element):
         brk = ColBreak.from_tree(element)
-        self.page_breaks.append(brk)
+        self.col_breaks = brk
+
+
+    def parse_custom_views(self, element):
+        # clear page_breaks to avoid duplication which Excel doesn't like
+        # basically they're ignored in custom views
+        self.row_breaks = RowBreak()
+        self.col_breaks = ColBreak()
 
 
 class WorksheetReader(object):
@@ -318,7 +344,9 @@ class WorksheetReader(object):
 
     def __init__(self, ws, xml_source, shared_strings, data_only):
         self.ws = ws
-        self.parser = WorkSheetParser(xml_source, shared_strings, data_only, ws.parent.epoch, ws.parent._date_formats)
+        self.parser = WorkSheetParser(xml_source, shared_strings,
+                data_only, ws.parent.epoch, ws.parent._date_formats,
+                ws.parent._timedelta_formats)
         self.tables = []
 
 
@@ -330,8 +358,9 @@ class WorksheetReader(object):
                 c._value = cell['value']
                 c.data_type = cell['data_type']
                 self.ws._cells[(cell['row'], cell['column'])] = c
-        self.ws.formula_attributes = self.parser.array_formulae
-        self.ws._current_row = self.parser.max_row
+
+        if self.ws._cells:
+            self.ws._current_row = self.ws.max_row # use cells not row dimensions
 
 
     def bind_formatting(self):
@@ -349,11 +378,17 @@ class WorksheetReader(object):
 
 
     def bind_merged_cells(self):
+        from openpyxl.worksheet.cell_range import MultiCellRange
+        from openpyxl.worksheet.merge import MergedCellRange
         if not self.parser.merged_cells:
             return
-        self.ws.merged_cells.ranges = self.parser.merged_cells.mergeCell
+
+        ranges = []
         for cr in self.parser.merged_cells.mergeCell:
-            self.ws._clean_merge_range(cr)
+            mcr = MergedCellRange(self.ws, cr.ref)
+            self.ws._clean_merge_range(mcr)
+            ranges.append(mcr)
+        self.ws.merged_cells = MultiCellRange(ranges)
 
 
     def bind_hyperlinks(self):
@@ -365,10 +400,24 @@ class WorksheetReader(object):
                 # range of cells
                 for row in self.ws[link.ref]:
                     for cell in row:
-                        cell.hyperlink = link
+                        try:
+                            cell.hyperlink = copy(link)
+                        except AttributeError:
+                            pass
             else:
-                self.ws[link.ref].hyperlink = link
+                cell = self.ws[link.ref]
+                if isinstance(cell, MergedCell):
+                    cell = self.normalize_merged_cell_link(cell.coordinate)
+                cell.hyperlink = link
 
+    def normalize_merged_cell_link(self, coord):
+        """
+        Returns the appropriate cell to which a hyperlink, which references a merged cell at the specified coordinates,
+        should be bound.
+        """
+        for rng in self.ws.merged_cells:
+            if coord in rng:
+                return self.ws.cell(*rng.top[0])
 
     def bind_col_dimensions(self):
         for col, cd in self.parser.column_dimensions.items():
@@ -390,7 +439,9 @@ class WorksheetReader(object):
         for k in ('print_options', 'page_margins', 'page_setup',
                   'HeaderFooter', 'auto_filter', 'data_validations',
                   'sheet_properties', 'views', 'sheet_format',
-                  'page_breaks', 'scenarios', 'legacy_drawing'):
+                  'row_breaks', 'col_breaks', 'scenarios', 'legacy_drawing',
+                  'protection',
+                  ):
             v = getattr(self.parser, k, None)
             if v is not None:
                 setattr(self.ws, k, v)
