@@ -1,7 +1,6 @@
-from __future__ import absolute_import
-# Copyright (c) 2010-2019 openpyxl
+# Copyright (c) 2010-2021 openpyxl
+
 
-from openpyxl.compat import unicode
 from openpyxl.descriptors.serialisable import Serialisable
 from openpyxl.descriptors import (
     Alias,
@@ -84,7 +83,6 @@ class SortState(Serialisable):
     def __bool__(self):
         return self.ref is not None
 
-    __nonzero__ = __bool__
 
 
 class IconFilter(Serialisable):
@@ -155,7 +153,7 @@ class CustomFilter(Serialisable):
 
     operator = NoneSet(values=(['equal', 'lessThan', 'lessThanOrEqual',
                             'notEqual', 'greaterThanOrEqual', 'greaterThan']))
-    val = String()
+    val = Float()
 
     def __init__(self,
                  operator=None,
@@ -175,7 +173,7 @@ class CustomFilters(Serialisable):
     __elements__ = ('customFilter',)
 
     def __init__(self,
-                 _and=None,
+                 _and=False,
                  customFilter=(),
                 ):
         self._and = _and
@@ -243,7 +241,7 @@ class Filters(Serialisable):
                                    "gregorianMeFrench","gregorianArabic", "hijri","hebrew",
                                    "taiwan","japan", "thai","korea",
                                    "saka","gregorianXlitEnglish","gregorianXlitFrench"])
-    filter = ValueSequence(expected_type=unicode)
+    filter = ValueSequence(expected_type=str)
     dateGroupItem = Sequence(expected_type=DateGroupItem, allow_none=True)
 
     __elements__ = ('filter', 'dateGroupItem')
@@ -334,7 +332,6 @@ class AutoFilter(Serialisable):
     def __bool__(self):
         return self.ref is not None
 
-    __nonzero__ = __bool__
 
 
     def add_filter_column(self, col_id, vals, blank=False):
