@@ -1,23 +1,18 @@
-from __future__ import absolute_import
-# Copyright (c) 2010-2019 openpyxl
+# Copyright (c) 2010-2021 openpyxl
 
 """Worksheet is the 2nd-level container in Excel."""
 
 
 # Python stdlib imports
-from itertools import islice, product, chain
+from itertools import chain
 from operator import itemgetter
 from inspect import isgenerator
+from warnings import warn
 
 # compatibility imports
 from openpyxl.compat import (
-    basestring,
     deprecated,
 )
-try:
-    range = xrange
-except NameError:
-    pass
 
 # package imports
 from openpyxl.utils import (
@@ -58,6 +53,8 @@ from .merge import MergedCellRange
 from .properties import WorksheetProperties
 from .pagebreak import RowBreak, ColBreak
 from .scenario import ScenarioList
+from .table import TableList
+from .formula import ArrayFormula
 
 
 class Worksheet(_WorkbookChild):
@@ -108,7 +105,6 @@ class Worksheet(_WorkbookChild):
                                                  default_factory=self._add_column)
         self.row_breaks = RowBreak()
         self.col_breaks = ColBreak()
-        self.page_breaks = [self.row_breaks, self.col_breaks]
         self._cells = {}
         self._charts = []
         self._images = []
@@ -116,7 +112,7 @@ class Worksheet(_WorkbookChild):
         self._drawing = None
         self._comments = []
         self.merged_cells = MultiCellRange()
-        self._tables = []
+        self._tables = TableList()
         self._pivots = []
         self.data_validations = DataValidationList()
         self._hyperlinks = []
@@ -133,7 +129,6 @@ class Worksheet(_WorkbookChild):
         self._current_row = 0
         self.auto_filter = AutoFilter()
         self.paper_size = None
-        self.formula_attributes = {}
         self.orientation = None
         self.conditional_formatting = ConditionalFormattingList()
         self.legacy_drawing = None
@@ -156,26 +151,29 @@ class Worksheet(_WorkbookChild):
     def active_cell(self):
         return self.sheet_view.selection[0].activeCell
 
+
+    @property
+    def array_formulae(self):
+        """Returns a dictionary of cells with array formulae and the cells in array"""
+        result = {}
+        for c in self._cells.values():
+            if c.data_type == "f":
+                if isinstance(c.value, ArrayFormula):
+                    result[c.coordinate] = c.value.ref
+        return result
+
+
     @property
     def show_gridlines(self):
         return self.sheet_view.showGridLines
 
 
-    """ To keep compatibility with previous versions"""
-    @property
-    def show_summary_below(self):
-        return self.sheet_properties.outlinePr.summaryBelow
-
-    @property
-    def show_summary_right(self):
-        return self.sheet_properties.outlinePr.summaryRight
-
-
     @property
     def freeze_panes(self):
         if self.sheet_view.pane is not None:
             return self.sheet_view.pane.topLeftCell
 
+
     @freeze_panes.setter
     def freeze_panes(self, topLeftCell=None):
         if isinstance(topLeftCell, Cell):
@@ -248,6 +246,8 @@ class Worksheet(_WorkbookChild):
         Internal method for getting a cell from a worksheet.
         Will create a new cell if one doesn't already exist.
         """
+        if not 0 < row < 1048577:
+            raise ValueError("Row numbers must be between 1 and 1048576")
         coordinate = (row, column)
         if not coordinate in self._cells:
             cell = Cell(self, row=row, column=column)
@@ -289,12 +289,12 @@ class Worksheet(_WorkbookChild):
         if not any([min_col, min_row, max_col, max_row]):
             raise IndexError("{0} is not a valid coordinate or range".format(key))
 
-        if not min_row:
+        if min_row is None:
             cols = tuple(self.iter_cols(min_col, max_col))
             if min_col == max_col:
                 cols = cols[0]
             return cols
-        if not min_col:
+        if min_col is None:
             rows = tuple(self.iter_rows(min_col=min_col, min_row=min_row,
                                         max_col=self.max_column, max_row=max_row))
             if min_row == max_row:
@@ -390,10 +390,7 @@ class Worksheet(_WorkbookChild):
         else:
             return "A1:A1"
 
-        return '%s%d:%s%d' % (
-            get_column_letter(min_col), min_row,
-            get_column_letter(max_col), max_row
-        )
+        return f"{get_column_letter(min_col)}{min_row}:{get_column_letter(max_col)}{max_row}"
 
 
     @property
@@ -430,7 +427,8 @@ class Worksheet(_WorkbookChild):
         """
 
         if self._current_row == 0 and not any([min_col, min_row, max_col, max_row ]):
-            return ()
+            return iter(())
+
 
         min_col = min_col or 1
         min_row = min_row or 1
@@ -496,7 +494,7 @@ class Worksheet(_WorkbookChild):
         """
 
         if self._current_row == 0 and not any([min_col, min_row, max_col, max_row]):
-            return ()
+            return iter(())
 
         min_col = min_col or 1
         min_row = min_row or 1
@@ -561,7 +559,21 @@ class Worksheet(_WorkbookChild):
 
 
     def add_table(self, table):
-        self._tables.append(table)
+        """
+        Check for duplicate name in definedNames and other worksheet tables
+        before adding table.
+        """
+
+        if self.parent._duplicate_name(table.name):
+            raise ValueError("Table with name {0} already exists".format(table.name))
+        if not hasattr(self, "_get_cell"):
+            warn("In write-only mode you must add table columns manually")
+        self._tables.add(table)
+
+
+    @property
+    def tables(self):
+        return self._tables
 
 
     def add_pivot(self, pivot):
@@ -570,22 +582,23 @@ class Worksheet(_WorkbookChild):
 
     def merge_cells(self, range_string=None, start_row=None, start_column=None, end_row=None, end_column=None):
         """ Set merge on a cell range.  Range is a cell range (e.g. A1:E1) """
-        cr = CellRange(range_string=range_string, min_col=start_column, min_row=start_row,
+        if range_string is None:
+            cr = CellRange(range_string=range_string, min_col=start_column, min_row=start_row,
                       max_col=end_column, max_row=end_row)
-        self.merged_cells.add(cr)
-        self._clean_merge_range(cr)
+            range_string = cr.coord
+        mcr = MergedCellRange(self, range_string)
+        self.merged_cells.add(mcr)
+        self._clean_merge_range(mcr)
 
 
-    def _clean_merge_range(self, cr):
+    def _clean_merge_range(self, mcr):
         """
         Remove all but the top left-cell from a range of merged cells
         and recreate the lost border information.
         Borders are then applied
         """
-        mcr = MergedCellRange(self, cr.coord)
-        cells = chain.from_iterable(mcr.rows)
+        cells = mcr.cells
         next(cells) # skip first cell
-
         for row, col in cells:
             self._cells[row, col] = MergedCell(self, row, col)
         mcr.format()
@@ -608,9 +621,8 @@ class Worksheet(_WorkbookChild):
 
         self.merged_cells.remove(cr)
 
-        cells = chain.from_iterable(cr.rows)
+        cells = cr.cells
         next(cells) # skip first cell
-
         for row, col in cells:
             del self._cells[(row, col)]
 
@@ -652,7 +664,7 @@ class Worksheet(_WorkbookChild):
 
         elif isinstance(iterable, dict):
             for col_idx, content in iterable.items():
-                if isinstance(col_idx, basestring):
+                if isinstance(col_idx, str):
                     col_idx = column_index_from_string(col_idx)
                 cell = Cell(self, row=row_idx, column=col_idx, value=content)
                 self._cells[(row_idx, col_idx)] = cell
@@ -753,7 +765,7 @@ class Worksheet(_WorkbookChild):
         Existing cells will be overwritten.
         Formulae and references will not be updated.
         """
-        if isinstance(cell_range, basestring):
+        if isinstance(cell_range, str):
             cell_range = CellRange(cell_range)
         if not isinstance(cell_range, CellRange):
             raise ValueError("Only CellRange objects can be moved")
@@ -870,7 +882,7 @@ class Worksheet(_WorkbookChild):
         """
         Range of cells in the form A1:D4 or list of ranges
         """
-        if isinstance(value, basestring):
+        if isinstance(value, str):
             value = [value]
 
         self._print_area = [absolute_coordinate(v) for v in value]
