@@ -1,4 +1,4 @@
-# Copyright (c) 2010-2019 openpyxl
+# Copyright (c) 2010-2021 openpyxl
 
 """
 XML compatability functions
@@ -7,15 +7,12 @@ XML compatability functions
 # Python stdlib imports
 import re
 from functools import partial
-# compatibility
 
-# package imports
 from openpyxl import DEFUSEDXML, LXML
 
-if LXML is True and DEFUSEDXML is False:
+if LXML is True:
     from lxml.etree import (
     Element,
-    ElementTree,
     SubElement,
     register_namespace,
     QName,
@@ -23,33 +20,26 @@ if LXML is True and DEFUSEDXML is False:
     XMLParser,
     )
     from lxml.etree import fromstring, tostring
-    from xml.etree.cElementTree import iterparse
     # do not resolve entities
     safe_parser = XMLParser(resolve_entities=False)
     fromstring = partial(fromstring, parser=safe_parser)
 
 else:
     from xml.etree.ElementTree import (
-    ElementTree,
     Element,
     SubElement,
     fromstring,
     tostring,
-    iterparse,
     QName,
     register_namespace
     )
-    if DEFUSEDXML is True:
-        from defusedxml.cElementTree import (
-        fromstring,
-        tostring,
-        iterparse,
-        )
     from et_xmlfile import xmlfile
+    if DEFUSEDXML is True:
+        from defusedxml.ElementTree import fromstring
 
-if LXML is True:
-    from lxml.etree import xmlfile # not affected by parsing vulns
-
+from xml.etree.ElementTree import iterparse
+if DEFUSEDXML is True:
+    from defusedxml.ElementTree import iterparse
 
 from openpyxl.xml.constants import (
     CHART_NS,
@@ -60,6 +50,7 @@ from openpyxl.xml.constants import (
     REL_NS,
     VTYPES_NS,
     COREPROPS_NS,
+    CUSTPROPS_NS,
     DCTERMS_NS,
     DCTERMS_PREFIX,
     XML_NS
@@ -76,6 +67,7 @@ register_namespace('vt', VTYPES_NS)
 register_namespace('xdr', SHEET_DRAWING_NS)
 register_namespace('cdr', CHART_DRAWING_NS)
 register_namespace('xml', XML_NS)
+register_namespace('cust', CUSTPROPS_NS)
 
 
 tostring = partial(tostring, encoding="utf-8")
